-- See Copyright Notice at the end of this file

-- allow NSObject to refer to the ObjC class

settagmethod(tag(nil), "getglobal", class)


-- allow object.selector(a,b,c), Class.selector(a,b,c) to work

function selectorLookup(table, selector)
 return function(...)
   tinsert(arg, 1, %table)
   tinsert(arg, 2, %selector)
   return call(%message, arg)
 end
end

settagmethod(tag(message(NSObject,"new")), "gettable", selectorLookup)
settagmethod(tag(NSObject), "gettable", selectorLookup)

-- nss: turn Lua strings into NSString objects
-- nsd: turn Lua strings into NSData objects

function nss(s) return NSString.stringWithCString_length_(s,strlen(s)) end
function nsd(s) return NSData.dataWithBytes_length_(s,strlen(s)) end

-- nsn: turn Lua numbers into NSNumber objects

function nsn(s) return NSNumber.numberWithDouble_(s) end

-- deeply clone a table

function deepclone(o, lookup)
 if type(o)~='table' then return o end
 lookup = lookup or {}
 if lookup[o] ~= nil then return lookup[o] end
 local new_o = {}
 lookup[o] = new_o
 local i,v for i,v in o do
  new_o[deepclone(i, lookup)] = deepclone(v, lookup)
 end
 local theTag = tag(o)
 if theTag ~= %tag({})then settag(new_o, tag(o)) end
 return new_o
end


function forwardInvocation(object, selector, invocation)
  local i, sig, argCount, v, args, bo
  bo = tableForBridgedObject(invocationArg(invocation,0))
  sel = invocationArg(invocation,1)
  args = { bo }
  sig = invocation.methodSignature()
  argCount = sig.numberOfArguments()
  for i = 2, argCount-1 do
    args[i] = invocationArg(invocation,i)
  end
  -- do invocation
  local r = call(bo[sel], args)
  setInvocationReturn(invocation, r)
end

-- Copyright (c) 2001 By Richard Kiss
--
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice,
--    this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright
--    notice, this list of conditions and the following disclaimer in the
--    documentation and/or other materials provided with the distribution.
-- 3. The name of the author may not be used to endorse or promote products
--    derived from this software without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
-- IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-- OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-- IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
-- EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
